@@ -13,7 +13,7 @@ import traceback
 from powershell_kernel import subprocess_repl, powershell_proxy
 from powershell_kernel.util import get_powershell
 
-__version__ = '0.1.2'
+__version__ = '0.1.3'
 
 version_pat = re.compile(r'version (\d+(\.\d+)+)')
 
@@ -61,8 +61,7 @@ class PowerShellKernel(Kernel):
             if not self.proxy:
                 self.__createProxy()
 
-            self.proxy.send_input('. { ' + code + ' }')
-            output = self.proxy.get_output()
+            output = self.proxy.run_command(code)
 
             message = {'name': 'stdout', 'text': output}
             self.send_response(self.iopub_socket, 'stream', message)
