@@ -3,7 +3,7 @@ try:
     import queue
 except ImportError:
     import Queue as queue
-from threading import Timer
+from threading import Timer, Lock
 from time import sleep
 
 class ReplReader(threading.Thread):
@@ -25,44 +25,46 @@ class ReplReader(threading.Thread):
 
 class ReplProxy(object):
     def __init__(self, repl):
+        self.runCmdLock = Lock()
+
         self._repl = repl
         self._repl_reader = ReplReader(repl)
-        # this is a hack to detect when we stop processing this input
-        self.send_input('function prompt() {"^"}')
 
         self.stop_flag = False
         self.output = ''
         self.timer = Timer(0.1, self.update_view_loop)
         self.timer.start()
-        # get preambula and eveluation of the prompt
-        self.get_output()
 
         self.output_prefix_stripped = True
         self.expected_output_prefix = ''
         self.expected_output_len = 0
 
+        # this is a hack to detect when we stop processing this input
+        self.run_command('function prompt() {"^"}')
 
-    def get_output(self):
-        while not self.stop_flag:
-            sleep(0.05)
-        out = self.output
-        self.output = ''
-        self.stop_flag = False 
-        return out       
+    def run_command(self, input):
+        self.runCmdLock.acquire()
+        try:
+            self.output = ''
+            self.stop_flag = False
 
-    def send_input(self, input):
-        # TODO: we should block here until we return output for previous command, should we?
+            # Append newline to the original input to handle single line comments on the last line
+            #
+            # Also, for multiline statements we should send 1 extra new line at the end
+            # https://stackoverflow.com/questions/13229066/how-to-end-a-multi-line-command-in-powershell
+            input = '. {\n' + input + '\n}\n'
 
-        # for multiline statements we should send 1 extra new line
-        # https://stackoverflow.com/questions/13229066/how-to-end-a-multi-line-command-in-powershell
-        if '\n' in input:
-            input += '\n'
+            self.expected_output_prefix = input.replace('\n', '\n>> ') + '\n'
+            self.expected_output_len = len(self.expected_output_prefix)
+            self.output_prefix_stripped = False
 
-        self.expected_output_prefix = input.replace('\n', '\n>> ') + '\n'
-        self.expected_output_len = len(self.expected_output_prefix)
-        self.output_prefix_stripped = False
+            self._repl.write(input + '\n')
 
-        self._repl.write(input + '\n')
+            while not self.stop_flag:
+                sleep(0.05)
+            return self.output
+        finally:
+            self.runCmdLock.release()
 
     def handle_repl_output(self):
         """Returns new data from Repl and bool indicating if Repl is still
@@ -95,14 +97,6 @@ class ReplProxy(object):
             return
         self.output += packet
 
-        if self.output_prefix_stripped and self.output == '>> ':
-            # this is most likely incomplete input problem
-            self.output = ("Incomplete input! PowerShell process would not be able to continue.\n"
-            "Due to the implementation limitation, you need to manually restart PowerShell Language Kernel.\n"
-            "Kernal -> Restart\n"
-            "For details, see https://github.com/vors/jupyter-powershell/issues/9")
-            self.stop_flag = True
-
         if not self.output_prefix_stripped and len(self.output) >= self.expected_output_len:
             if self.output[:self.expected_output_len] != self.expected_output_prefix:
                 print("Unexpected prefix: %r : Expected %r" % (
